# -*- encoding: utf-8 -*-
# $Id$
##
# copyright 2008, James William Pye. http://jwp.name
##
import unittest
import name.jwp.ri as ri

def cmbx(t):
	'Yield a list of combinations using a mask'
	tl = len(t)
	return [
		[x & m and t[x-1] or '' for x in xrange(1, tl + 1)]
		for m in xrange(1, (2 ** tl) + 1)
	]

def sample_strings(chars):
	for x in (unichr(c) for c in xrange(0x10000)):
		for y in (unichr(c1) for c1 in xrange(0x10000)):
			for z in (unichr(c2) for c2 in xrange(0x10000)):
				yield x + y + z

expectation_samples = (
	(
		'',
		(None, None, None, None, None),
		{}
	),
	(
		'host',
		(None, 'host', None, None, None),
		{
			'host' : 'host',
		}
	),
	(
		'http://host',
		('http', 'host', None, None, None),
		{
			'scheme' : 'http',
			'host' : 'host',
		}
	),
	(
		'http://host/',
		('http', 'host', '', None, None),
		{
			'scheme' : 'http',
			'host' : 'host',
			'path' : [],
		}
	),
	(
		'http://host/path1',
		('http', 'host', 'path1', None, None),
		{
			'scheme' : 'http',
			'host' : 'host',
			'path' : ['path1'],
		}
	),
	(
		'http://host/path1/',
		('http', 'host', 'path1/', None, None),
		{
			'scheme' : 'http',
			'host' : 'host',
			'path' : ['path1',''],
		}
	),
	(
		'http://host/path1/path2',
		('http', 'host', 'path1/path2', None, None),
		{
			'scheme' : 'http',
			'host' : 'host',
			'path' : ['path1','path2'],
		}
	),
	(
		'http://host/path1/path2?k=v&k2=v2',
		('http', 'host', 'path1/path2', 'k=v&k2=v2', None),
		{
			'scheme' : 'http',
			'host' : 'host',
			'path' : ['path1','path2'],
			'query' : [('k', 'v'), ('k2', 'v2')],
		}
	),
	(
		'http://host/path1/path2?k=v',
		('http', 'host', 'path1/path2', 'k=v', None),
		{
			'scheme' : 'http',
			'host' : 'host',
			'path' : ['path1','path2'],
			'query' : [('k', 'v')],
		}
	),
	(
		'http://host/path1/path2?k=v#',
		('http', 'host', 'path1/path2', 'k=v', ''),
		{
			'scheme' : 'http',
			'host' : 'host',
			'path' : ['path1','path2'],
			'query' : [('k', 'v')],
			'fragment' : '',
		}
	),
	(
		'http://host/path1/path2?k=v#fragment',
		('http', 'host', 'path1/path2', 'k=v', 'fragment'),
		{
			'scheme' : 'http',
			'host' : 'host',
			'path' : ['path1','path2'],
			'query' : [('k', 'v')],
			'fragment' : 'fragment',
		}
	),
	(
		'http://host:port/path1/path2?k=v#fragment',
		('http', 'host:port', 'path1/path2', 'k=v', 'fragment'),
		{
			'scheme' : 'http',
			'host' : 'host',
			'port' : 'port',
			'path' : ['path1','path2'],
			'query' : [('k', 'v')],
			'fragment' : 'fragment',
		}
	),
	(
		'http://user@host:port/path1/path2?k=v#fragment',
		('http', 'user@host:port', 'path1/path2', 'k=v', 'fragment'),
		{
			'scheme' : 'http',
			'user' : 'user',
			'host' : 'host',
			'port' : 'port',
			'path' : ['path1','path2'],
			'query' : [('k', 'v')],
			'fragment' : 'fragment',
		}
	),
	(
		'http://user:pass@host:port/path1/path2?k=v#fragment',
		('http', 'user:pass@host:port', 'path1/path2', 'k=v', 'fragment'),
		{
			'scheme' : 'http',
			'user' : 'user',
			'password' : 'pass',
			'host' : 'host',
			'port' : 'port',
			'path' : ['path1','path2'],
			'query' : [('k', 'v')],
			'fragment' : 'fragment',
		}
	),
	(
		'ftp://user:pass@host:port/pa%2Fth1/path2?#',
		('ftp', 'user:pass@host:port', 'pa%2Fth1/path2', '', ''),
		{
			'scheme' : 'ftp',
			'user' : 'user',
			'password' : 'pass',
			'host' : 'host',
			'port' : 'port',
			'path' : ['pa/th1','path2'],
			'query' : [],
			'fragment' : '',
		}
	),
	(
		'ftp://us%40er:pa:ss@host:port/pa%2Fth1/path2?#',
		('ftp', 'us%40er:pa:ss@host:port', 'pa%2Fth1/path2', '', ''),
		{
			'scheme' : 'ftp',
			'user' : 'us@er',
			'password' : 'pa:ss',
			'host' : 'host',
			'port' : 'port',
			'path' : ['pa/th1','path2'],
			'query' : [],
			'fragment' : '',
		}
	),
	(
		'ftp://us%40er:pa:ss@host:port/pa%2Fth1/path2?%23=%23#',
		('ftp', 'us%40er:pa:ss@host:port', 'pa%2Fth1/path2', '%23=%23', ''),
		{
			'scheme' : 'ftp',
			'user' : 'us@er',
			'password' : 'pa:ss',
			'host' : 'host',
			'port' : 'port',
			'path' : ['pa/th1','path2'],
			'query' : [('#', '#')],
			'fragment' : '',
		}
	),
)

sample_unsplit_netlocs = (
	'sample.com',
	'host',
	'host:port',
	'host:8080',
	'@host',
	'%40@',
	'%40:@host',
	'user@',
	'user:@',
	'user:pass@',
	':@host',
	':@host:8080',
	'user:@host:8080',
	'user:@[]:8080',
	'user:@[::1]:8080',
	'user:@[text]:8080',
	'user:@[:8080',
	'user:@[fe80::1]:8080',
	'user:pass@host:8080',
	'user:pass@host',
)

sample_split_netlocs = (
	('user', 'pass', 'host', 'port'),
	('@', ':', 'host', 'port'),
	('@user', 'pass:', 'host', 'port'),
	('@user', 'pass:', 'host', ''),
	('@user', 'pass:', '', ''),
	('user', '@', u'host.com', ''),
	('user', '@', u'å.com', '8080'),
	('user', '', u'x--na.com', '8080'),
	('', '', u'x--na.com', '8080'),
	('', '', u'', '8080'),
	('', '', u'', ''),
)

sample_unsplit_paths = (
	None,
	'',
	'/',
	'///',
	'/path',
	'/path/path2',
	'/path/path2/',
	'/path2',
	u'/åß∂',
	u'/åß∂/',
	u'/åß∂//',
	'/element1/element2/element3',
)

sample_schemes = (
	'',
	'http',
	'x-un+known',
)

sample_users = (
	'',
	':',
	'@',
	'user@',
	'@user',
	'user',
	u'jæms',
)

sample_passwords = [
	'',
	':',
	'@',
	':pass',
	'pass:',
	'pass',
	u'æçèêí',
]

sample_hosts = [
	'',
	'[::1]',
	#'[::1', Fail case. Can't use this host and correctly identify the port.
	#'::1]', Fail case. Can't use this host and correctly identify the port.
	'10.0.0.1',
	'localhost',
	'remotehost.tld',
]

sample_ports = [
	'',
	'80',
	'foo',
	':',
]

sample_paths = (
	[],
	# [''], Fail case. Can't represent this structure consistently.
	['', ''],
	['/',],
	['/', '/', '/'],
	['//',],
	['//', '//'],
	['path','path2','path3'],
	['path/'],
	['path/','path2'],
	['path','/path2'],
	['/path/','pa/th2','/path3',],
)

sample_queries = (
	[],
	# [('', None)], Fail case. Can't represent this structure consistently.
	[('key', None)],
	[('key', None), ('key2', None)],
	[('key', 'val')],
	[('key', 'val'), ('key', None)],
	[('?', '&')],
	[('&', '&')],
	[('=', '&')],
	[('=', '=')],
)

sample_fragments = (
	'',
	'#',
	'fragment#',
	'\x05',
	'%05',
)

def samples():
	for s in sample_schemes:
		for u in sample_users:
			for p in sample_passwords:
				for h in sample_hosts:
					for pt in sample_ports:
						for path in sample_paths:
							for q in sample_queries:
								for f in sample_fragments:
									r = {
										'scheme' : s,
										'user' : u,
										'password' : p,
										'host' : h,
										'port' : pt,
										'path' : path,
										'query' : q,
										'fragment' : f,
									}
									yield r
									rr = r.copy()
									del rr['host']
									del rr['port']
									yield rr

									rr = r.copy()
									del rr['query']
									yield rr

									rr = r.copy()
									del rr['path']
									yield rr

									rr = r.copy()
									del rr['scheme']
									yield rr

									rr = r.copy()
									del rr['user']
									del rr['password']
									yield rr

									rr = r.copy()
									del rr['fragment']
									yield rr

class test_name_jwp_ri(unittest.TestCase):
	def testCombinations(self):
		for x in samples():
			s = ri.serialize(x)
			p = ri.parse(s)
			if p != x:
				self.fail("%r -> %r != %r" %(x, s, p))

	def testExpectations(self):
		for x in expectation_samples:
			text, split, parsed = x

			text_split = ri.split(text)
			text_parsed = ri.parse(text)

			split_unsplit = ri.unsplit(split)
			split_structure = ri.structure(split)

			parsed_construct = ri.construct(parsed)
			parsed_serialize = ri.serialize(parsed)

			self.failUnless(
				text_parsed == parsed,
				"missed parse expectation: %r -> %r != %r" %(
					text, text_parsed, parsed
				)
			)
			self.failUnless(
				parsed_serialize == text,
				"missed serialize expectation: %r -> %r != %r" %(
					parsed, parsed_serialize, text
				)
			)
			self.failUnless(
				parsed_construct == split,
				"missed parsed_construct expectation: %r -> %r != %r" %(
					parsed, parsed_construct, split
				)
			)
			self.failUnless(
				split_unsplit == text,
				"missed split_unsplit expectation: %r -> %r != %r" %(
					split, split_unsplit, text
				)
			)
			self.failUnless(
				split_structure == parsed,
				"missed split_structure expectation: %r -> %r != %r" %(
					split, split_structure, parsed
				)
			)
			self.failUnless(
				text_split == split,
				"missed text_split expectation: %r -> %r != %r" %(
					text, text_split, split
				)
			)

	def testSplitUnsplitNetloc(self):
		for x in sample_unsplit_netlocs:
			sn = ri.split_netloc(x)
			usn = ri.unsplit_netloc(sn)
			self.failIf(
				usn != x,
				"[netloc] split-unsplit incongruent: %r -> %r != %r" %(
					x, sn, usn
				)
			)

	def testUnsplitSplitNetloc(self):
		for xx in sample_split_netlocs:
			for x in cmbx(xx):
				x = tuple(x)
				un = ri.unsplit_netloc(x)
				sn = tuple(ri.split_netloc(un))
				self.failIf(
					sn != x,
					"[netloc] unsplit-split incongruent: %r -> %r != %r" %(
						x, un, sn
					)
				)

	def testSplitUnsplitPath(self):
		for x in sample_unsplit_paths:
			s = ri.split_path(x)
			us = ri.unsplit_path(s)
			self.failIf(
				us != x,
				"[path] split-unsplit incongruent: %r -> %r -> %r != %r" %(
					x, s, us, x
				)
			)

	def testUnsplitSplitPath(self):
		for x in sample_paths:
			us = ri.unsplit_path(x)
			s = ri.split_path(us)
			self.failIf(
				s != x,
				"[path] unsplit-split incongruent: %r -> %r -> %r != %r" %(
					x, us, s, x
				)
			)

if __name__ == '__main__':
	from types import ModuleType
	this = ModuleType("this")
	this.__dict__.update(globals())
	unittest.main(this)
